options(echo=FALSE)

#This file is used to create Table 3 of the manuscript (the power function of the test statistic for the gamma example)
#If run as is (simply execute the code without changing anything) the output (it takes about 20 hours on a intel core i7 870 PC, using windows 7 and R version 2.14) will be identical to Table 3
#the random seed is controled by the set.seed() command in row  204 below
#For experimenting with the code, the inputs are:

# 1) cut-off points - are calculated seperately by running the code on file "Critical.Values.Gamma.Table3.R" (it is ready to run) and then should be copy pasted in line 200 below. N.B.: for 100,000 iterations and for the sample sizes considered here the procedure has been done already, i.e. the user does not need to do anything
# 2) Specify the H_0 and H_1 distributions: The options supported so far are "expon" for exponential, "weib" for weibull, "gamma" for gamma - this is done on line 211
# 3) Specify the distribution parameters in line 207 (current version uses values so that the distance between H_0 and H_1 is increasignly deviant with respect to Kullback-Liebler ), th user is free to set their own parameters.


"Bandwidth"<-function(sample)
  #Rule of thumb bandwidth plug-in rules
  #input: sample data
  #output: a vector of two components:
  #Component 1 is the density bandwidth rule of Silverman (Density estimation for statistics and data analysis, Chapman and Hall, London, 1986), eq. 3.3.1
  #Component 2 is the distribution function bandwidth rule of Bowman, Halland Prvan (1988) (Bandwidth selection for the smoothing of distribution functions, Biometrika, Vol 85, page 801)
{
  n<-length(sample)
  StD<-sd(sample)
  IqR<-diff(quantile(sample, c(.25, .75)))
  tmp<-min(StD, IqR/1.34)
  c( 0.9*tmp*n^(-1/5) , 3.572* tmp * n^(-1/3)) #first entry is density simple plug in rule, second entry is distribution function simple plug in rule
}

"Epanechnikov"<-
  function(x){
    #Epanechikov kernel, square root of 5 is taken as 2.236068 (to save comp. time).
    ifelse(abs(x)< 2.236068, (3/4)*(1-((x^2)/5 ))/2.236068, ifelse(abs(x)>2.236068, 0,0))
  }


"IntEpanechnikov"<-
  function(x){ #Integrated Epanechikov kernel, square root of 5 is taken as 2.236068 (to save comp. time).
    ifelse(x< -2.236068, 0, ifelse(x> 2.236068, 1, .5- (2.236068*x*(x^2-15))/100))
  }


"HStar"<-function(xin, cens, xout)
  #Kaplan-Meier Estimate, slightly modified so that it avoids division by zero.
  #In: sample and censoring indicators, i.e. xin and cens correspond to $(Z_i, \Lambda_i)$ in manuscript 
  #(in fact the first two lines of the routine ensure that this will be the case anyway). The xout argument denotes the points where the estimate will be evaluated
  #the most common option is to use xin in the place of xout
  #Out: 1-\hat H^\star(X_i) (as given on the manuscript)
{
  ordxin <-  sort(xin) #order input data=Xi
  ordcens <- cens[order(xin)] #order censoring =Di
  n <- length(xin)
  nn <- length(xout)
  vKuse <- sapply(1:nn, function (i, ordxin, xout) max(c(max(which(ordxin <= xout[i]))
                                                         , min(which(xout[i] <= ordxin)))), ordxin, xout)
  vK <- vKuse - 1
  Huse<-sapply(vK, function (i,n,ordcens) ((n-1:i+1)/(n-1:i+2))^(1-ordcens[1:i]), n, ordcens)
  result<-sapply(Huse, prod)
  result[order(xin)]<-result
  result
}


"GPPlugInBand"<-function(xin, Di)
  #Gulati and Padgett (1996) (Families of smooth confidence bands for the survival function under the general random censorship model, Lifetime Data Anal., 2 349-362)
  #survival function, MSE optimal bandwidth selection rule (see equation 7 in the manuscript)
  #In: sample and censoring indicators
  #Out: vector of bandwidth values (ready to use in the \hat S_n estimate)
{
  n<-length(xin)
  Mu2sq<- 1 #specifically for Epanechnikov
  intxKW<-0.2874945 #specifically for Epanechnikov
  ThetaHat<-sum(xin)/sum(Di) #estimate theta for reference distribution
  arg1Use <- exp(-xin/ThetaHat ) #the body of the f,f' definition is evaluated once only to save comp. time
  FHat<- 1/ThetaHat * arg1Use #\tilde f
  FHatDer<- -1/ThetaHat^2 * arg1Use #\tilde f'
  KMUse<- HStar(xin , Di, xin ) #kaplan-meier
  
  ((2*FHat * intxKW )/ (n*KMUse * FHatDer^2 * Mu2sq))^(1/3) #apply the definition and return result 
}

"MPDensityPlugInBand"<-function(xin, Di)
  #Marron and Padgett (1986) (Asymptotically optimal bandwidth selection for kernel density estimators from randomly right censored samples, Ann. Statist, 15, 1520-1535)
  #density function, MSE optimal bandwidth selection rule (see formula for h_0 in the manuscript)
  #In: sample and censoring indicators
  #Out: vector of bandwidth values (ready to use in the f_n estimate)
{
  n<-length(xin)
  RK<-  (3/25) * sqrt(5) #specifically for Epanechnikov
  ThetaHat<-sum(xin)/sum(Di) #estimate theta for reference distribution
  arg1Use <- exp(-xin/ThetaHat )#the body of the f,f' definition is evaluated once only to save comp. time
  FHat<- 1/ThetaHat * arg1Use  #\tilde f
  FHatSecondDer<- 1/ThetaHat^3 * arg1Use #\tilde f'' 
  KMUse<- HStar(xin , Di, xin ) #kaplan-meier
  
  ((RK*FHat / KMUse )/ ( 4*( 1/2)^2 * n* FHatSecondDer^2 ))^(1/5)  #apply the definition and return result
}

"TestStatistic"<-function(xin, Di, xout,  kfun, Intkfun, H0dist, H1dist, p1,p2, p3,p4)
{
  #The test statistic #\tilde T_{n,\ast} / \sigma(\tilde T_{n,\ast})
  #in: sample (xin),  censoring indicators (Di), xout (set equal to xin), kfun (Epabechnikov kernel), Intkfun (Integrated Epanechnikov kernel)
  #    H0dist (Distribution under the null), H1dist (alternative distribution), p1,p2 (paramaters for the null distribution), p3,p4 (parameters under the alternative)
  #out: value of the test statistic
  TrueSurv<-switch(H1dist, gamma=1-pgamma(xin, shape=p3,scale=p3), weib=1-pweibull(xin, shape=p3,scale=p3), expon=1-pexp(xin, rate=p3))
  Rw<- (26/35) * sqrt(5) #specifically for Epanechnikov
  RK<- (3/25) * sqrt(5) #specifically for Epanechnikov
  
  k<-1/2   ## k =\mu_2^2(K)/2 #specifically for Epanechnikov
  s<-  10.36478   #this is the S functinal in the paper, specifically for Epanechnikov
  n<-length(xin)
  KaplanMeier <- HStar(xin , Di , xin ) # Kaplan-Meier
  arg1<-Di/KaplanMeier #D_i/KM - used in the \hat S_n, f_n definitions
  h<-GPPlugInBand(xin, Di) #GP MSE bandwidth for the survival function estimate
  xXiDif<-(matrix(xin , ncol=n, nrow=n, byrow=T) - xin)#for the argument of the kernel
  xXiDifUse<-(matrix(-xin , ncol=n, nrow=n, byrow=T) - xin) # to be used for reflection in the density estimate - Jones (1993)
  out <- colSums(Intkfun(xXiDif /h ) ) #calculate \sum_{i=1}^n W(x-X_i)/h
  hatSn <- 1-out/n  #calculate \hat S_n
  h<-MPDensityPlugInBand(xin, Di)*0.45 #f_n optimal bandwidth 
  outdesn <-  colSums(kfun(xXiDif/h) * arg1 + kfun(xXiDifUse/h) * arg1	  ) #calculate \sum_{i=1}^n K(x-X_i)/h + K(-x-X_i)/h (i.e.with reflection)
  hatfn <-  outdesn/(n * h ) #f_n estimate
  h2<-  (4/(n*(1+2*1+2)))^(1/(1+2*1+4))*sd(xin) #derivative optimal bandwidth (simple plugin rule)
  hatfnder<- colSums(xXiDif/h2 * kfun(xXiDif/h2)  * arg1  +  xXiDifUse/h2 * kfun(xXiDifUse/h2)  * arg1) # apply equation 10 in manuscript (with reflection)
  hatfnder<- hatfnder/(h2^2 * n * 2*k) # 2*k because k=\mu_2^2/2 and s2 in Jones (1993) paper is s2=\mu_2^2. Also: the - sign from the estimate is removed because of S''_n(x) = -f'_n(x)
  # \tilde T_{n,\ast}:
  h<-Bandwidth(xin)[2] #get distribution function bandwidth for the statistic
  hatfnOverKaplanMeier <-hatfn/KaplanMeier
  hatsigma2 <- ( h*   Rw *  sum(hatfnOverKaplanMeier ) - h^2   * RK * sum( (1-hatSn )^2 ))/n^2 #calculate \sigma_2^2
  Tn<-  (sum( (hatSn  - TrueSurv)^2 )- h^2 *k * sum(hatfnder^2))/n - hatsigma2  #caluclate \tilde T_{n,\ast}
  #Standard deviation  \tilde T_{n,\ast}:
  s1sq<- sum(hatfnOverKaplanMeier^2) #calculate \hat \sigma_1
  s4sq<- sum(hatfnder^2 * hatfnOverKaplanMeier - hatfnder^2 *(1-hatSn ))#calculate \hat \sigma_4
  lamhat<- n*h^3
  stdevTn<- (s1sq *sqrt(2*lamhat) + 2* lamhat^(1/3) * s4sq * k )* (s/n) #apply equation 11 in manuscript
  
  TestStatisticVal<-  n^(3/2) * Tn/stdevTn #apply the d(n) factor
  TestStatisticVal #return the test statistic value
}



"All.TestValues"<-function(data, cen,  kfun, Intkfun, H0dist, H1dist, p1,p2, p3,p4)
  # return the test values for the vector p3 of distribution parameters
{
  cUse1<-length(p3)
  AllTestValues<-sapply(1:cUse1, function(j, data, cen,  kfun, Intkfun, H0dist, H1dist, p1,p2, p3,p4)
        {
         p3use<-p3[j]
         TestStatistic(data, cen, data,  kfun, Intkfun, H0dist, H1dist, p1,p2, p3use,p4)
        },data, cen,  kfun, Intkfun, H0dist, H1dist, p1,p2, p3,p4 )
  AllTestValues
}


PowerFunction<-function(ItNumber, SampleSizeVector, CensParamVector, CritValsVector, H0dist, H1dist, kfun, Intkfun,  p1,p2, p3,p4)
{
  #Implementation of the power function (equation 14, manuscript)
  #in: ItNumber (number of iterations over which the power function is calculated, currently this is set to 10,000), 
  #SampleSizeVector (defines the sample sizes for which the power of the statistic is claculated), 
  #CensParamVector (the censoring percentage of the sample), 
  #CritValsVector (the cut-off points of the statistic, calculated externally), 
  #H0dist (Distribution under the null), H1dist (alternative distribution), p1,p2 (paramaters for the null distribution), p3,p4 (parameters under the alternative)
  #out: matrix with rejection percentages under H_0 and the 12 variants of H_1
  m<-length(SampleSizeVector)
  cUse1<-length(p3)
  PowerMatOut<-matrix(nrow=m, ncol=1+cUse1) #initialize the matrix which holds the rejection percentages
  for(i in 1:m) #define the combinations of sample size and censoring level and select appropriate cut-off point to apply
  {
    SampleSizeUse<-SampleSizeVector[i] #select the sample size
    CensParamuse<-CensParamVector[i] #select censoring percentage
    CritValsUse<-CritValsVector[i] #select appropriate cut-off point
    SampleIndices<- 1:SampleSizeUse
    TestValues<-sapply(1:ItNumber, function(i, SampleSizeUse, CensParamuse, H0dist, H1dist, kfun, Intkfun, p1,p2, p3,p4,cUse1, SampleIndices)
    {
      data<-switch(H0dist, weib=rweibull(SampleSizeUse, p1,p2), gamma=rgamma(SampleSizeUse, shape=p1,scale=p2), expon=rexp(SampleSizeUse)) #generate data from appropriate distribution
      CensInd<-sample(SampleIndices, CensParamuse*SampleSizeUse/100) #select the indices of the sample which are considered censored
      cen <- rep(1, times=length(data))
      cen[CensInd]<-0 # censored observations are marked with 0
      cUse<- CensParamuse #100* length(which(data<cens))/SampleSizeUse
      AllTestValues<- All.TestValues(data, cen,  kfun, Intkfun, H0dist, H1dist, p1,p2, p3,p4) #calculate test values for H_0 and the 12 variants of H_1 
      c(cUse,   AllTestValues) #return the resulting test statistic values as well as censoring percentage used				
    }, SampleSizeUse, CensParamuse, H0dist, H1dist, kfun, Intkfun, p1,p2,p3,p4, cUse1, SampleIndices)
    
    TestValues<-matrix(TestValues, ncol=1+cUse1, byrow=T) #put the above result in matrix format
    PowerMatOut[i,1]<- TestValues[1]
    TestValues<-TestValues[,c(2:ncol(TestValues))]
    dout<-vector(length=ncol(TestValues))
    for(k in 1:ncol(TestValues))
    { 
      TestValUse<- TestValues[,k]
      
      dout[k] <- 100*length(TestValUse[which(TestValUse>CritValsUse)])/ItNumber #apply the power function definition
    }
    PowerMatOut[i,c(2:ncol(PowerMatOut))]<- dout #store the rejection percentage in the appropriate place in the matrix
  }
  PowerMatOut #return the matrix with all rejection percentages
}



SampleSizeVector<-rep(c(30,  40,	50,	60,	70,	80,	90,	100),4) #define the sample sizes (30-100), replicate 4 times, one for each censoring percentage
CensParamVector <- rep((0:3)*10,each=8) #define the censoring percentages of the samples, so that they match the sample sizes and all unique combinations are created
critic.vals<- c(  -0.2495395  ,  -0.3551386 , -0.4802218  ,  -0.6137422, -0.7548498,   -0.9032818,	-1.060836 ,	 -1.223125  )#based on 100000 sample values,calculated in ... file

CritValsVector<- rep(critic.vals, 4) #replicate so that they match the sample size/censoring combinations

set.seed(157) #use this to get exactly the same numbers as in the paper
ItNumber<-10000 #number of iterations over which the rejection percentages are calculated

DistParameters<-c(1, 1.05,1.1,1.15,1.2,1.25,1.3,1.35,1.4,1.45,1.5,1.55, 1.6 ,1.65) #specify the null and 12 H_1 variants distribution parameters

ResultsMat<-matrix(nrow = 4*8, ncol=2*length(DistParameters)) #initialize output matrix

arg<-PowerFunction(ItNumber, SampleSizeVector, CensParamVector, CritValsVector, "gamma", "gamma", Epanechnikov, IntEpanechnikov,  1,1, DistParameters,1) #call the power function calculation routine
ResultsMat<-arg #store the rejection percenatages in the matrix created previously
SScol<-SampleSizeVector
MyDistName<-paste("g(", DistParameters, ")") #assign names to the matrix columns
ResultsMat<- cbind( SScol, ResultsMat) # bind the sample sizes to the power matrix (used for pretty printing)
a<-c( "samplesize", "Cens", MyDistName[1:length(DistParameters)]) #formulate the names of the columns of the power matrix (used for pretty printing)
colnames(ResultsMat)<-a #assign the column names to the power matrix (used for pretty printing)

ResultsMat #return the final result
